% Thermodynamic analysis
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Markovian Dynamics on Reaction Networks       %%%%%%%%%%%%
%%%%%%%%%%%             Example: Neural dynamics               %%%%%%%%%%%%
%%%%%%%%%%%              THERMODYNAMIC ANALYSIS                %%%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias             %%%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 04/24/2012             %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
                 
tf       = 1000;           % simulation time (in msec)
timeStep = 0.5;            % time step (in msec)
L        = 100;            % total number of neurons (even number)
time     = 0:timeStep:tf;  % simulation time array

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve master equation by KSA (no avalanching)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[y1y2probs,Q,probs] = neural(1,tf,timeStep,L);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute thermodynamic quantities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Compute energy surface
%

E1 = -log(y1y2probs(:,:,end));

%
% Compute internal energy 
%

U1 = zeros(length(time),1);
for t = 1:length(time)
    U1(t) = sum(sum(E1.*y1y2probs(:,:,t)));
end

%
% Compute entropy
%

S1 = zeros(length(time),1);

for t = 1:length(time)
    temporary = log(y1y2probs(:,:,t)).*y1y2probs(:,:,t);
    S1(t)      = -sum(sum( temporary(~isnan(temporary))));
end

%
% Compute Helmholtz free energy
%

F1 = U1-S1;

%
% Compute derivatives by finite differencing
%

dS_dt = zeros(length(S1),1);
dF_dt = zeros(length(F1),1);

for t = 1:length(F1)-1
    dS_dt(t) = (S1(t+1)-S1(t))/timeStep;
    dF_dt(t) = (F1(t+1)-F1(t))/timeStep;
end

dS_dt(end) = dS_dt(end-1);
dF_dt(end) = dF_dt(end-1);

%
% Compute entropy production rate
%

entProd1 = zeros(length(time),1);

for i=1:length(probs)
    for j=1:length(probs)
        if (Q(i,j)~=0 && Q(j,i)~=0)
            for t=1:length(time)
                temporary = .5*(Q(j,i)*probs(i,t)-Q(i,j)*probs(j,t))...
                            .*log((Q(j,i)*probs(i,t))./(Q(i,j)*probs(j,t)));
                if ~isinf(temporary)&&~isnan(temporary)                      
                    entProd1(t) = entProd1(t) + temporary;
                end
            end
        end
    end
end

%
% Compute heat dissipation rate and motive power
%

heatDiss1 = -dS_dt + entProd1;
motPower1 =  dF_dt + entProd1;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve master equation by KSA (avalanching)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[y1y2probs,Q,probs] = neural(2,tf,timeStep,L);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute thermodynamic quantities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% Compute energy surface
%

E2 = -log(y1y2probs(:,:,end));

%
% Compute internal energy 
%

U2 = zeros(length(time),1);
for t = 1:length(time)
    U2(t) = sum(sum(E2.*y1y2probs(:,:,t)));
end

%
% Compute entropy
%

S2 = zeros(length(time),1);

for t = 1:length(time)
    temporary = log(y1y2probs(:,:,t)).*y1y2probs(:,:,t);
    S2(t)     = -sum(sum( temporary(~isnan(temporary))));
end

%
% Compute Helmholtz free energy
%

F2 = U2-S2;

%
% Compute derivatives by finite differencing
%

dS_dt = zeros(length(S2),1);
dF_dt = zeros(length(F2),1);

for t = 1:length(F2)-1
    dS_dt(t) = (S2(t+1)-S2(t))/timeStep;
    dF_dt(t) = (F2(t+1)-F2(t))/timeStep;
end

dS_dt(end) = dS_dt(end-1);
dF_dt(end) = dF_dt(end-1);

%
% Compute entropy production rate
%

entProd2 = zeros(length(time),1);

for i=1:length(probs)
    for j=1:length(probs)
        if (Q(i,j)~=0 && Q(j,i)~=0)
            for t=1:length(time)
                temporary = .5*(Q(j,i)*probs(i,t)-Q(i,j)*probs(j,t))...
                            .*log((Q(j,i)*probs(i,t))./(Q(i,j)*probs(j,t)));
                if ~isinf(temporary)&&~isnan(temporary)                      
                    entProd2(t) = entProd2(t) + temporary;
                end
            end
        end
    end
end

%
% Compute heat dissipation rate and motive power
%

heatDiss2 = -dS_dt + entProd2;
motPower2 =  dF_dt + entProd2;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make and save plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

lnw=1.5;

fig1 = figure('Position',[50 50 650 700]);
h1   = subplot(1,1,1);
surf(h1,0:(L/2),0:(L/2),E1)
set(h1,'XGrid','on');
set(h1,'YGrid','on');
set(h1,'ZGrid','off');
colormap('jet')
shading('interp')
axis('tight')
title('\bf{energy landscape (no avalanching)}');
xlabel('{\bf active excitatory neurons}');
ylabel('{\bf active inhibitory neurons}');
zlabel('E({\bf y})');

fig2 = figure('Position',[50 50 650 700]);
h2   = subplot(1,1,1);
surf(h2,0:(L/2),0:(L/2),E2)
set(h2,'XGrid','on');
set(h2,'YGrid','on');
set(h2,'ZGrid','off');
colormap('jet')
shading('interp')
axis('tight')
title('\bf{energy landscape (avalanching)}');
xlabel('{\bf active excitatory neurons}');
ylabel('{\bf active inhibitory neurons}');
zlabel('E({\bf y})');

figure;
fig3 = plot(time,F1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,F2,'LineWidth',lnw,'Color','red');
axis([0 200 0 16]);
xlabel('time (msec)');
ylabel('Helmoltz free energy');
legend('no avalanching','avalanching',1);

figure;
fig4 = plot(time,U1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,U2,'LineWidth',lnw,'Color','red');
axis([0 200 0 16]);
xlabel('time (msec)');
ylabel('internal energy');
legend('no avalanching','avalanching',1);

figure;
fig5 = plot(time,S1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,S2,'LineWidth',lnw,'Color','red');
axis([0 200 0 7]);
xlabel('time (msec)');
ylabel('entropy');
legend('no avalanching','avalanching',1);     

figure;
fig6 = plot(time,entProd1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,entProd2,'LineWidth',lnw,'Color','red');
axis([0 200 0 2.5]);
xlabel('time (msec)');
ylabel('entropy production rate');
legend('no avalanching','avalanching',1); 

figure;
fig7 = plot(time,heatDiss1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,heatDiss2,'LineWidth',lnw,'Color','red');
axis([0 200 0 2.5]);
xlabel('time (msec)');
ylabel('heat dissipation rate');
legend('no avalanching','avalanching',1); 

figure;
fig8 = plot(time,motPower1,'LineWidth',lnw,'Color','blue');
hold on
plot(time,motPower2,'LineWidth',lnw,'Color','red');
axis([0 200 0 2.5]);
xlabel('time (msec)');
ylabel('supplied motive power');
legend('no avalanching','avalanching',1);       

saveas(fig1,'results/energyNoAval.fig');
saveas(fig2,'results/energyAval.fig');
saveas(fig3,'results/freeEnergy.fig');
saveas(fig4,'results/intEnergy.fig');
saveas(fig5,'results/entropy.fig');
saveas(fig6,'results/epr.fig');
saveas(fig7,'results/hdr.fig');
saveas(fig8,'results/smp.fig');