%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Markovian Dynamics on Reaction Networks       %%%%%%%%%%%%
%%%%%%%%%%%           Example: Neural dynamics                 %%%%%%%%%%%%
%%%%%%%%%%%              THERMODYNAMIC ANALYSIS                %%%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias             %%%%%%%%%%%%
%%%%%%%%%%%             Last Modified: 04/17/2012              %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function [y1y2probs,Q,probs] = neural(parameterSet,tf,timeStep,L)

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

h     = 0.001;
gamma = 0.1;

if parameterSet==1     
    nu_E_plus_nu_I  = 0.004; 
    nu_E_minus_nu_I = 0.00276;
else
    nu_E_plus_nu_I  = 0.004; 
    nu_E_minus_nu_I = 0.276;
end

nu_I = 0.5*(nu_E_plus_nu_I+nu_E_minus_nu_I);
nu_E = 0.5*(nu_E_plus_nu_I-nu_E_minus_nu_I);

numStates = ((L/2)+1)*((L/2)+1);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tol = 1e-6;   % EXPOKIT tolerance parameter
K0  = 40;     % Krylov subspace dimension

probs     = zeros(((L/2)+1)*((L/2)+1),tf/timeStep); % probabilties over time
y1y2probs = zeros((L/2)+1,(L/2)+1,tf/timeStep);     % probabilties over time in y1y2 plane

display('Running Expokit. Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make generator matrix of Markov process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

row   = zeros(5*numStates,1);
col   = zeros(5*numStates,1);
value = zeros(5*numStates,1);

count = 1;
for y1plusOne = 1:((L/2)+1)     % number of excitory neurons
    for y2plusOne = 1:((L/2)+1) % number of inhibitory neurons
        
        %
        % find propensities
        %
        
        y1 = y1plusOne-1;
        y2 = y2plusOne-1;
        
        phi = nu_E*y1+nu_I*y2+h; % synaptic input to neurons
        
        propensity(1) = ((L/2)-y1)*max(tanh(phi),0);
        propensity(2) = gamma*y1;
        propensity(3) = ((L/2)-y2)*max(tanh(phi),0);
        propensity(4) = gamma*y2;
            
        %
        % add value to diagonal
        %
        
        diagVal      = (y1plusOne-1)*((L/2)+1)+y2plusOne;
        row(count)   = diagVal;
        col(count)   = diagVal;
        value(count) = -1*(propensity(1)+propensity(2)+propensity(3) ...
                          +propensity(4));
        count        = count+1;

        %
        % add propensity for reaction 1
        %

        if propensity(1)>0
            % the location with one more y1
            location1    = (y1plusOne)*((L/2)+1)+y2plusOne;            
            row(count)   = location1;
            col(count)   = diagVal;
            value(count) = propensity(1);
            count        = count+1;
        end

        %
        % add propensity for reaction 2
        %

        if propensity(2)>0
            % the location with one less y1
            location2    = (y1plusOne-2)*((L/2)+1)+y2plusOne;            
            row(count)   = location2;
            col(count)   = diagVal;
            value(count) = propensity(2);
            count        = count+1;
        end
        
        %
        % add propensity for reaction 3
        %

        if propensity(3)>0
            % the location with one more y2
            location3    = (y1plusOne-1)*((L/2)+1)+y2plusOne+1;            
            row(count)   = location3;
            col(count)   = diagVal;
            value(count) = propensity(3);
            count        = count+1;
        end
        
        %
        % add propensity for reaction 4
        %

        if propensity(4)>0
            % the location with one less y2
            location4    = (y1plusOne-1)*((L/2)+1)+y2plusOne-1;            
            row(count)   = location4;
            col(count)   = diagVal;
            value(count) = propensity(4);
            count        = count+1;
        end
        
    end
end

Q = sparse(row(value~=0),col(value~=0),value(value~=0),numStates,numStates);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set initial condition to all neurons being inactive
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

p0                                    = zeros(size(Q,1),1);
y1plusOne                             = 1;
y2plusOne                             = 1;
p0((y1plusOne-1)*((L/2)+1)+y2plusOne) = 1;
pPrev                                 = p0;
timeCount                             = 1;  

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Map probability to y1-y2 plane
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

probs(:,timeCount) = pPrev;
count = 1;
for y1plusOne = 1:((L/2)+1)
    for y2plusOne = 1:((L/2)+1)
        y1y2probs(y1plusOne,y2plusOne,timeCount)=pPrev(count);
        count = count+1;
    end
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use EXPOKIT software package
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for t= timeStep:timeStep:tf

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Use EXPOKIT to take timestep
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
   
    pNext     = mexpv(timeStep,Q,pPrev,tol,K0);
    pPrev     = pNext;
    timeCount = timeCount+1;
    
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Map probability to y1-y2 plane and store result
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    probs(:,timeCount)=pNext;
    count = 1;
    for y1plusOne = 1:((L/2)+1)
        for y2plusOne = 1:((L/2)+1)
            y1y2probs(y1plusOne,y2plusOne,timeCount)=pNext(count);
            count = count+1;
        end
    end
    
end