%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Markovian Dynamics on Reaction Networks       %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%        KRYLOV SUBSPACE APPROXIMATION METHOD        %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 04/18/2012                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet = 1; % 1 is for monostable system (liberal system)
                  % 2 is for bistable system (totalitarian system)
                
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if parameterSet==1 % monostable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 0/L;
    a2     = .5/L;
    a3     = .5/L;
else % bistable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 1.5/L;
    a2     = 1/L;
    a3     = -.25/L;
end

numStates = (2*L+1)*(2*L+1);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tf        = 15;   % simulation time in days
timeStep  = .1;   % time step in days
tol       = 1e-6; % EXPOKIT tol parameter
K0        = 40;   % Krylov subspace dimension
t         = 0;    % time variable

x1x2probs = zeros(2*L+1,2*L+1,tf/timeStep); % probabilties over time

tic;

display('Running Krylov subspace approximation method. Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make generator matrix of Markov process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

row=zeros(5*numStates,1);
col=zeros(5*numStates,1);
value=zeros(5*numStates,1);
count=1;
for x1plusLplusOne = 1:(2*L+1)
    for x2plusLplusOne = 1:(2*L+1)
        
        %
        % find propensities
        %
        x1 = x1plusLplusOne-L-1;
        x2 = x2plusLplusOne-L-1;
        
        propensity(1)=kappa1*(L-x1)*exp(a1*x1+a2*x2);
        propensity(2)=kappa1*(L+x1)*exp(-a1*x1-a2*x2);
        propensity(3)=kappa2*(L-x2)*exp(a3*x1);
        propensity(4)=kappa2*(L+x2)*exp(-a3*x1);
        
    
        %
        % add value to diagonal
        %
        diagVal      = (x1plusLplusOne-1)*(2*L+1)+x2plusLplusOne;
        row(count)   = diagVal;
        col(count)   = diagVal;
        value(count) = -1*(propensity(1)+propensity(2)+propensity(3) ...
                          +propensity(4));
        count        = count+1;

        %
        % add propensity for reaction 1
        %

        if propensity(1)>0
            % the location with one more x1
            location1    = (x1plusLplusOne)*(2*L+1)+x2plusLplusOne;            
            row(count)   = location1;
            col(count)   = diagVal;
            value(count) = propensity(1);
            count        = count+1;
        end

        %
        % add propensity for reaction 2
        %

        if propensity(2)>0
            % the location with one less x1
            location2    = (x1plusLplusOne-2)*(2*L+1)+x2plusLplusOne;            
            row(count)   = location2;
            col(count)   = diagVal;
            value(count) = propensity(2);
            count        = count+1;
        end
        
        %
        % add propensity for reaction 3
        %

        if propensity(3)>0
            % the location with one more x2
            location3    = (x1plusLplusOne-1)*(2*L+1)+x2plusLplusOne+1;            
            row(count)   = location3;
            col(count)   = diagVal;
            value(count) = propensity(3);
            count        = count+1;
        end
        
        %
        % add propensity for reaction 4
        %

        if propensity(4)>0
            % the location with one less x2
            location4    = (x1plusLplusOne-1)*(2*L+1)+x2plusLplusOne-1;            
            row(count)   = location4;
            col(count)   = diagVal;
            value(count) = propensity(4);
            count        = count+1;
        end
        
    end
end

Q = sparse(row(value~=0),col(value~=0),value(value~=0),numStates,numStates);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set initial condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

p0             = zeros(size(Q,1),1);
x1plusLplusOne = 0+L+1;
x2plusLplusOne = 0+L+1;
p0((x1plusLplusOne-1)*(2*L+1)+x2plusLplusOne)=1;
pPrev          = p0;
timeCount      = 1;  

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Map probability to x1-x2 plane
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
count=1;
for x1plusLplusOne = 1:(2*L+1)
    for x2plusLplusOne = 1:(2*L+1)
        x1x2probs(x1plusLplusOne,x2plusLplusOne,timeCount)=pPrev(count);
        count=count+1;
    end
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% USE EXPOKIT software package
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for t= timeStep:timeStep:tf

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use EXPOKIT to take timestep
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
    [pNext,err,hump] = mexpv(timeStep,Q,pPrev,tol,K0);
    pPrev=pNext;
    timeCount=timeCount+1;
    
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Map probability to x1-x2 plane and store result
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    
    count=1;
    for x1plusLplusOne = 1:(2*L+1)
        for x2plusLplusOne = 1:(2*L+1)
            x1x2probs(x1plusLplusOne,x2plusLplusOne,timeCount)=pNext(count);
            count=count+1;
        end
    end
    
end

toc;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Marginal and Joint Stationary Distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

x1x2StatProbs = zeros(2*L+1,2*L+1);
count=1;
for x1plusLplusOne = 1:(2*L+1)
    for x2plusLplusOne = 1:(2*L+1)
        x1x2StatProbs(x1plusLplusOne,x2plusLplusOne)=pNext(count);
        count=count+1;
    end
end

if parameterSet == 1
    fig = figure('Position',[50 50 650 700]);
    h1 = subplot(1,1,1);
    surf(h1,-L:1:L,-L:1:L,x1x2StatProbs)
    set(h1,'XGrid','on');
    set(h1,'YGrid','on');
    set(h1,'ZGrid','off');
    colormap('jet')
    shading('interp')
    axis('tight')
    title('\bf{Krylov subspace approximation}');
    xlabel('net private opinion');
    ylabel('net public opinion');
    zlabel('probability');
    saveas(fig,'results/KrylovJoint_monoStable.fig');
else
    fig = figure('Position',[50 50 650 700]);
    h1 = subplot(1,1,1);
    surf(h1,-L:1:L,-L:1:L,x1x2StatProbs)
    set(h1,'XGrid','on');
    set(h1,'YGrid','on');
    set(h1,'ZGrid','off');
    colormap('jet')
    shading('interp')
    axis('tight')
    title('\bf{Krylov subspace approximation}');
    xlabel('net private opinion');
    ylabel('net public opinion');
    zlabel('probability');
    saveas(fig,'results/KrylovJoint_biStable.fig');
end

figure;
bar(-L:1:L,sum(x1x2StatProbs,2));
axis([-40  40 0 1.1*max(sum(x1x2StatProbs,2))])
title('\bf{Krylov subspace approximation}');
xlabel('net public opinion');
ylabel('probability');
colormap white
if parameterSet == 1
    saveas(gcf,'results/KrylovMargPub_monoStable.fig')
else
    saveas(gcf,'results/KrylovMargPub_biStable.fig')
end

figure;
bar(-L:1:L,sum(x1x2StatProbs,1));
axis([-40  40 0 1.1*max(sum(x1x2StatProbs,1))])
title('\bf{Krylov subspace approximation}');
xlabel('net private opinion');
ylabel('probability');
colormap white
if parameterSet == 1
    saveas(gcf,'results/KrylovMargPriv_monoStable.fig')
else
    saveas(gcf,'results/KrylovMargPriv_biStable.fig')
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute and plot the means and standard devs from movie
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
   
%
% Compute marginals over time
%
x1probs = squeeze(sum(x1x2probs,2));
x2probs = squeeze(sum(x1x2probs,1));

%
% Compute means
%
x1mean = zeros(1,size(x1probs,2));
x2mean = zeros(1,size(x2probs,2));
count = 1;
for x=-L:1:L
    x1mean = x1mean + x*x1probs(count,:);
    x2mean = x2mean + x*x2probs(count,:);
    count = count+1;
end

%
% Compute variances
%
x1var = zeros(1,size(x1probs,2));
x2var = zeros(1,size(x2probs,2));
count = 1;
for x=-L:1:L
    x1var = x1var + (x-x1mean).^2.*x1probs(count,:);
    x2var = x2var + (x-x2mean).^2.*x2probs(count,:);
    count = count+1;
end

%
% Generate figs
%

lnw=1.5;

figure;
plot(0:timeStep:tf,x1mean,'r','LineWidth',lnw); hold on;
plot(0:timeStep:tf,x1mean+sqrt(x1var),':','LineWidth',lnw); hold on;
plot(0:timeStep:tf,x1mean-sqrt(x1var),':','LineWidth',lnw); 
title('\bf{Krylov subspace approximation}');
xlabel('time (days)');
ylabel('net public opinion');
axis([0 tf -40 40])
if parameterSet == 1
    saveas(gcf,'results/KrylovMeanPub_monoStable.fig')
else
    saveas(gcf,'results/KrylovMeanPub_biStable.fig')
end

figure;
plot(0:timeStep:tf,x2mean,'r','LineWidth',lnw); hold on;
plot(0:timeStep:tf,x2mean+sqrt(x2var),':','LineWidth',lnw); hold on;
plot(0:timeStep:tf,x2mean-sqrt(x2var),':','LineWidth',lnw);
title('\bf{Krylov subspace approximation}');
xlabel('time (days)');
ylabel('net private opinion');
axis([0 tf -40 40])
if parameterSet == 1
    saveas(gcf,'results/KrylovMeanPriv_monoStable.fig')
else
    saveas(gcf,'results/KrylovMeanPriv_biStable.fig')
end