%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%           MAXENT APPROXIMATION METHOD              %%%%%%%%%%%%
%%%%%%%%%%%           Authors: Jenkinson/Goutsias              %%%%%%%%%%%%
%%%%%%%%%%%            Last Modified: 04/18/2012               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

parameterSet=1; % 1 is for monostable system (liberal democracy)
                % 2 is for bistable system (totalitarian state)

if parameterSet==1 % monostable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 0/L;
    a2     = .5/L;
    a3     = .5/L;
else % bistable system
    L      = 40;
    kappa1 = 20/L;
    kappa2 = 40/L;
    a1     = 1.5/L;
    a2     = 1/L;
    a3     = -.25/L;
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% INPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

mciter    =  4000;                  % NUMBER OF MONTE CARLO ITERATIONS
M         =  4;                     % NUMBER OF REACTIONS
N         =  2;                     % NUMBER OF SPIECES
totalTime =  15;                    % SIMULATION TIME (IN DAYS)
dt        =  0.1;                   % SAMPLING RATE (IN DAYS)

xinit = [0 0]';

if parameterSet == 1
    fidz = fopen('results/averagez_monoStable.dat','w');
else
    fidz = fopen('results/averagez_biStable.dat','w');
end

display('Running MaxEnt approximation method. Please wait...');
tic;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

for i=1:mciter
    rand(1);
    memory = 1e6;
    Z   = repmat(0,M,1);
    ZZ  = repmat(0,M,memory);
    iter = 1;
    for t=dt:dt:totalTime
        iter  = iter + 1;
        alpha(1)  = kappa1*(L-(xinit(1)+Z(1)-Z(2)))*...
                      exp(a1*(xinit(1)+Z(1)-Z(2))+a2*(xinit(2)+Z(3)-Z(4)));
        alpha(2)  = kappa1*(L+(xinit(1)+Z(1)-Z(2)))*...
                      exp(-a1*(xinit(1)+Z(1)-Z(2))-a2*(xinit(2)+Z(3)-Z(4)));
        alpha(3)  = kappa2*(L-(xinit(2)+Z(3)-Z(4)))*exp(a3*(xinit(1)+Z(1)-Z(2)));
        alpha(4)  = kappa2*(L+(xinit(2)+Z(3)-Z(4)))*exp(-a3*(xinit(1)+Z(1)-Z(2)));

        alpha(alpha<0) = 0;  
       
        R          = poissrnd(alpha.*dt);
        Z          = Z + R';
        ZZ(:,iter) = Z;
    end 
    ZZZ = ZZ(:,1:iter);
    B   = ZZZ(:,1:1:totalTime/dt);
    fwrite(fidz,B,'double');
end

fclose(fidz);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MOMENT COMPUTATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

Zdata = repmat(0,[mciter,M,totalTime/dt]);
Xdata = repmat(0,[mciter,N,totalTime/dt]);

if parameterSet == 1
    fidz = fopen('results/averagez_monoStable.dat','r');
else
    fidz = fopen('results/averagez_biStable.dat','r');
end

for i=1:mciter
    Zdata(i,:,:)=fread(fidz,[M,totalTime/dt],'double');
end
fclose(fidz);

for i=1:mciter
    Xdata(i,1,1:totalTime/dt)=xinit(1)+Zdata(i,1,1:totalTime/dt)...
                                          -Zdata(i,2,1:totalTime/dt);
    Xdata(i,2,1:totalTime/dt)=xinit(2)+Zdata(i,3,1:totalTime/dt)...
                                          -Zdata(i,4,1:totalTime/dt);  
end

steadyStateX1 = Xdata(:,1,end);
steadyStateX2 = Xdata(:,2,end);

momentsX1 = [mean(steadyStateX1);mean(steadyStateX1.^2);...
             mean(steadyStateX1.^3);mean(steadyStateX1.^4);...
             mean(steadyStateX1.^5);mean(steadyStateX1.^6);...
             mean(steadyStateX1.^7);mean(steadyStateX1.^8)];
momentsX2 = [mean(steadyStateX2);mean(steadyStateX2.^2);...             
             mean(steadyStateX2.^3);mean(steadyStateX2.^4);...
             mean(steadyStateX2.^5);mean(steadyStateX2.^6);...            
             mean(steadyStateX2.^7);mean(steadyStateX2.^8)];
         
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% COMPUTE MAXENT DISTRIBUTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% MaxEnt calculations using second- and fourth-order moments
%

xmin=-40; xmax=40; dx=1; % define the x axis
x=(xmin:dx:xmax)';

[lambda1_2m,p1_2m,entr1_2m] = maxent(momentsX1(1:2),x);
[lambda2_2m,p2_2m,entr2_2m] = maxent(momentsX2(1:2),x);

[lambda1_4m,p1_4m,entr1_4m] = maxent(momentsX1(1:4),x);
[lambda2_4m,p2_4m,entr2_4m] = maxent(momentsX2(1:4),x);

[lambda1_8m,p1_8m,entr1_8m] = maxent(momentsX1(1:8),x);
[lambda2_8m,p2_8m,entr2_8m] = maxent(momentsX2(1:8),x);

toc

figure;
bar(x,p1_2m);
colormap white
axis([-L L 0 1.1*max(p2_2m)])
xlabel('net public opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (second-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT2margPub_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT2margPub_biStable.fig');
end

figure;
bar(x,p2_2m);
colormap white
axis([-L L 0 1.1*max(p2_2m)])
xlabel('net private opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (second-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT2margPriv_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT2margPriv_biStable.fig');
end

figure;
bar(x,p1_4m);
colormap white
axis([-L L 0 1.1*max(p1_4m)])
xlabel('net public opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (fourth-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT4margPub_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT4margPub_biStable.fig');
end

figure;
bar(x,p2_4m);
colormap white
axis([-L L 0 1.1*max(p2_4m)])
xlabel('net private opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (fourth-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT4margPriv_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT4margPriv_biStable.fig');
end

figure;
bar(x,p1_8m);
colormap white
axis([-L L 0 1.1*max(p1_8m)])
xlabel('net public opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (eighth-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT8margPub_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT8margPub_biStable.fig');
end

figure;
bar(x,p2_8m);
colormap white
axis([-L L 0 1.1*max(p2_8m)])
xlabel('net private opinion');
ylabel('probability');
title('{\bf MaxEnt approximation (eighth-order)}');
if parameterSet==1 % monostable system
    saveas(gcf,'results/MAXENT8margPriv_monoStable.fig');
else %bistable system
    saveas(gcf,'results/MAXENT8margPriv_biStable.fig');
end