%exact Monte Carlo sampling
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%       Markovian Dynamics on Reaction Networks       %%%%%%%%%%%
%%%%%%%%%%%          Example: Transcription Regulation          %%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias              %%%%%%%%%%%
%%%%%%%%%%%             Last Modified: 03/30/2012               %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear all;
close all;

iter = 2000;  % number of Monte Carlo iterations

fid   = fopen('results/exact.dat','w');
wtbar = waitbar(0,sprintf('Exact Monte Carlo sampling. Please wait ...'));

for i=1:iter
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%% INPUT PARAMETERS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
	V         =  1e-15;                    % CELLULAR VOLUME
	kappa1    =  0.043;                    % SPC OF TRANSLATION
	kappa2    =  0.083;                    % SPC OF PROTEIN BINDING
	kappa3    =  0.5;                      % SPC OF PROTEIN UNBINDING
	kappa4    =  0.0199;                   % SPC OF DNA BINDING AT R1
	kappa5    =  0.4791;                   % SPC OF DNA UNBINDING AT R1
	kappa6    =  1.9926e-4;                % SPC OF DNA BINDING AT R2
	kappa7    =  8.7658e-12;               % SPC OF DNA UNBINDING AT R2
	kappa8    =  0.0715;                   % SPC OF TRANSCRIPTION
	kappa9    =  0.0039;                   % SPC OF mRNA DEGRADATION
	kappa10   =  0.0007;                   % SPC OF PROTEIN DEGRADATION
	M         =  10;                       % NUMBER OF REACTIONS
	N         =  6;                        % NUMBER OF SPIECES
	pmono     =  2;                        % INITIAL NUMBER OF MONOMERS
	pdim      =  4;                        % INITIAL NUMBER OF DIMERS
	dna       =  2;                        % GENE COPY NUMBER
	totalTime =  35;                       % SIMULATION TIME (IN MINUTES)
	
	xinit     = [pmono pdim 0 dna 0 0]';
    totalTime = totalTime*60;
    time      = (0:totalTime-1);

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%% MONTE CARLO ITERATIONS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    clear alpha t tt X XX XXX Z ZZ 
    waitbar((i-1)/iter,wtbar);
    rand(1);
    memory = 500000;
	t      = 0;
	tt     = nan(1,memory);
    tt(1)  = t;
    zinit  = zeros(M,1);
    Z      = zinit;
	ZZ     = zinit;
    X      = xinit;
    XX     = nan(N,memory);
	XX(:,1)= xinit;
    count  = 1;
    
    while t < totalTime
        count     = count+1; 
        alpha(1)  = kappa1*(Z(8)-Z(9));
        alpha(2)  = kappa2*(pmono+Z(1)-Z(10)-2*Z(2)+2*Z(3))...
                          *(pmono+Z(1)-2*Z(2)+2*Z(3)-Z(10)-1)/2; 
        alpha(3)  = kappa3*(pdim+Z(2)-Z(3)-Z(4)+Z(5)-Z(6)+Z(7)); 
        alpha(4)  = kappa4*(pdim+Z(2)-Z(3)-Z(4)+Z(5)-Z(6)+Z(7))...
                          *(dna-Z(4)+Z(5)); 
        alpha(5)  = kappa5*(Z(4)-Z(5)-Z(6)+Z(7));
        alpha(6)  = kappa6*(pdim+Z(2)-Z(3)-Z(4)+Z(5)-Z(6)+Z(7))...
                          *(Z(4)-Z(5)-Z(6)+Z(7));
        alpha(7)  = kappa7*(Z(6)-Z(7));
        alpha(8)  = kappa8*(Z(4)-Z(5)-Z(6)+Z(7));
        alpha(9)  = kappa9*(Z(8)-Z(9));
        alpha(10) = kappa10*(pmono+Z(1)-2*Z(2)+2*Z(3)-Z(10));
        astr      = sum(alpha);
        if ~astr
            t = totalTime;
        else
            tau  = exprnd(inv(astr));
            m    = find(cumsum(alpha) > astr*rand, 1 );
            t    = t + tau;
            Z(m) = Z(m) + 1;
        end
        X(1) = Z(8)-Z(9);
        X(2) = pmono+Z(1)-Z(10)-2*Z(2)+2*Z(3);
        X(3) = pdim+Z(2)-Z(3)-Z(4)+Z(5)-Z(6)+Z(7);
        X(4) = dna-Z(4)+Z(5);
        X(5) = Z(4)-Z(5)-Z(6)+Z(7);
        X(6) = Z(6)-Z(7);
        tt(:,count) = t;
        XX(:,count) = X;
    end 
    %
    % Store results of current iteration
    %
    St = find(isnan(tt)>0,1);
    if isempty(St)
        St = length(tt)+1;
    end
    XXX(1,:) = interp1(tt(1:(St-1)),XX(1,1:(St-1)),time,'nearest');
    XXX(2,:) = interp1(tt(1:(St-1)),XX(2,1:(St-1)),time,'nearest');
    XXX(3,:) = interp1(tt(1:(St-1)),XX(3,1:(St-1)),time,'nearest');
    XXX(4,:) = interp1(tt(1:(St-1)),XX(4,1:(St-1)),time,'nearest');
    XXX(5,:) = interp1(tt(1:(St-1)),XX(5,1:(St-1)),time,'nearest');
    XXX(6,:) = interp1(tt(1:(St-1)),XX(6,1:(St-1)),time,'nearest');
    fwrite(fid,XXX,'double');
end

fclose(fid);
close(wtbar);